@@ -8,7 +8,7 @@
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
@@ -26,21 +26,22 @@
 
 #ifdef __cplusplus
 extern "C" {
+#pragma }
 #endif /* __cplusplus */
 
 
-#define GTK_ADJUSTMENT(obj)          GTK_CHECK_CAST (obj, gtk_adjustment_get_type (), GtkAdjustment)
-#define GTK_ADJUSTMENT_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_adjustment_get_type (), GtkAdjustmentClass)
-#define GTK_IS_ADJUSTMENT(obj)       GTK_CHECK_TYPE (obj, gtk_adjustment_get_type ())
+#define GTK_ADJUSTMENT(obj)	     (GTK_CHECK_CAST (obj, gtk_adjustment_get_type (), GtkAdjustment))
+#define GTK_ADJUSTMENT_CLASS(klass)  (GTK_CHECK_CLASS_CAST (klass, gtk_adjustment_get_type (), GtkAdjustmentClass))
+#define GTK_IS_ADJUSTMENT(obj)	     (GTK_CHECK_TYPE (obj, gtk_adjustment_get_type ()))
 
 
-typedef struct _GtkAdjustment       GtkAdjustment;
+typedef struct _GtkAdjustment	    GtkAdjustment;
 typedef struct _GtkAdjustmentClass  GtkAdjustmentClass;
 
 struct _GtkAdjustment
 {
   GtkData data;
-
+  
   gfloat lower;
   gfloat upper;
   gfloat value;
@@ -52,21 +53,25 @@ struct _GtkAdjustment
 struct _GtkAdjustmentClass
 {
   GtkDataClass parent_class;
-
-  void (* changed)       (GtkAdjustment *adjustment);
+  
+  void (* changed)	 (GtkAdjustment *adjustment);
   void (* value_changed) (GtkAdjustment *adjustment);
 };
 
 
-guint      gtk_adjustment_get_type   (void);
-GtkObject* gtk_adjustment_new        (gfloat              value,
-				      gfloat              lower,
-				      gfloat              upper,
-				      gfloat              step_increment,
-				      gfloat              page_increment,
-				      gfloat              page_size);
-void       gtk_adjustment_set_value  (GtkAdjustment      *adjustment,
-				      gfloat              value);
+GtkType	   gtk_adjustment_get_type   (void);
+GtkObject* gtk_adjustment_new	     (gfloat		  value,
+				      gfloat		  lower,
+				      gfloat		  upper,
+				      gfloat		  step_increment,
+				      gfloat		  page_increment,
+				      gfloat		  page_size);
+void	   gtk_adjustment_set_value  (GtkAdjustment	 *adjustment,
+				      gfloat		  value);
+void	   gtk_adjustment_clamp_page (GtkAdjustment	 *adjustment,
+				      gfloat		  lower,
+				      gfloat		  upper);
+     
 
 
 
