@@ -1394,7 +1394,7 @@ sync_selection (GtkCList * clist,
   list = clist->selection;
   while (list)
     {
-      if ((gint) list->data >= row)
+      if (GPOINTER_TO_INT (list->data) >= row)
         switch (mode)
           {
           case SYNC_INSERT:
@@ -2520,7 +2520,7 @@ draw_row (GtkCList * clist,
       if (!gdk_rectangle_intersect (area, &row_rectangle, &intersect_rectangle))
 	return;
 
-      if (clist_row->state == GTK_STATE_SELECTED || clist_row->fg_set)
+      if (clist_row->state == GTK_STATE_SELECTED || clist_row->bg_set)
 	gdk_draw_rectangle (clist->clist_window,
 			    bg_gc,
 			    TRUE,
@@ -2559,7 +2559,7 @@ draw_row (GtkCList * clist,
 			      cell_rectangle.height);     
 	}	  
 
-      if (clist_row->state == GTK_STATE_SELECTED || clist_row->fg_set)
+      if (clist_row->state == GTK_STATE_SELECTED || clist_row->bg_set)
 	gdk_draw_rectangle (clist->clist_window,
 			    bg_gc,
 			    TRUE,
@@ -2679,6 +2679,26 @@ draw_row (GtkCList * clist,
 	  ydest = (clip_rectangle.y + (clip_rectangle.height / 2)) - height / 2 +
 	    clist_row->cell[i].vertical;
 
+	  if (xdest < clip_rectangle.x)
+	    {
+	      xsrc = clip_rectangle.x - xdest;
+	      pixmap_width -= xsrc;
+	      xdest = clip_rectangle.x;
+	    }
+
+	  if (xdest + pixmap_width > clip_rectangle.x + clip_rectangle.width)
+	    pixmap_width = (clip_rectangle.x + clip_rectangle.width) - xdest;
+
+	  if (ydest < clip_rectangle.y)
+	    {
+	      ysrc = clip_rectangle.y - ydest;
+	      height -= ysrc;
+	      ydest = clip_rectangle.y;
+	    }
+
+	  if (ydest + height > clip_rectangle.y + clip_rectangle.height)
+	    height = (clip_rectangle.y + clip_rectangle.height) - ydest;
+
           if (GTK_CELL_PIXMAP (clist_row->cell[i])->mask)
           {
               gdk_gc_set_clip_mask (fg_gc, GTK_CELL_PIXMAP (clist_row->cell[i])->mask);
@@ -2707,6 +2727,26 @@ draw_row (GtkCList * clist,
 	  ydest = (clip_rectangle.y + (clip_rectangle.height / 2)) - height / 2 +
 	    clist_row->cell[i].vertical;
 
+	  if (xdest < clip_rectangle.x)
+	    {
+	      xsrc = clip_rectangle.x - xdest;
+	      pixmap_width -= xsrc;
+	      xdest = clip_rectangle.x;
+	    }
+
+	  if (xdest + pixmap_width > clip_rectangle.x + clip_rectangle.width)
+	    pixmap_width = (clip_rectangle.x + clip_rectangle.width) - xdest;
+
+	  if (ydest < clip_rectangle.y)
+	    {
+	      ysrc = clip_rectangle.y - ydest;
+	      height -= ysrc;
+	      ydest = clip_rectangle.y;
+	    }
+
+	  if (ydest + height > clip_rectangle.y + clip_rectangle.height)
+	    height = (clip_rectangle.y + clip_rectangle.height) - ydest;
+
           if (GTK_CELL_PIXTEXT (clist_row->cell[i])->mask)
           {
               gdk_gc_set_clip_mask (fg_gc, GTK_CELL_PIXTEXT (clist_row->cell[i])->mask);
@@ -3061,7 +3101,7 @@ real_select_row (GtkCList * clist,
   if (clist_row->state == GTK_STATE_NORMAL)
     {
       clist_row->state = GTK_STATE_SELECTED;
-      clist->selection = g_list_append (clist->selection, (gpointer) row);
+      clist->selection = g_list_append (clist->selection, GINT_TO_POINTER (row));
 
       if (!GTK_CLIST_FROZEN (clist)
 	  && (gtk_clist_row_is_visible (clist, row) != GTK_VISIBILITY_NONE))
@@ -3087,7 +3127,7 @@ real_unselect_row (GtkCList * clist,
   if (clist_row->state == GTK_STATE_SELECTED)
     {
       clist_row->state = GTK_STATE_NORMAL;
-      clist->selection = g_list_remove (clist->selection, (gpointer) row);
+      clist->selection = g_list_remove (clist->selection, GINT_TO_POINTER (row));
 
       if (!GTK_CLIST_FROZEN (clist)
 	  && (gtk_clist_row_is_visible (clist, row) != GTK_VISIBILITY_NONE))
