@@ -24,8 +24,8 @@
 #include "gtksignal.h"
 
 
-#define OBJECT_DATA_ID_CHUNK  1024
-
+#define GTK_OBJECT_DATA_ID_BLOCK_SIZE	(1024)
+#define	GTK_OBJECT_DATA_BLOCK_SIZE	(1024)
 
 enum {
   DESTROY,
@@ -74,7 +74,6 @@ static void           gtk_object_real_destroy  (GtkObject      *object);
 static void           gtk_object_finalize      (GtkObject      *object);
 static void           gtk_object_notify_weaks  (GtkObject      *object);
 
-static void           gtk_object_data_destroy  (GtkObjectData  *odata);
 static guint*         gtk_object_data_id_alloc (void);
 
 GtkArg*               gtk_object_collect_args  (guint   *nargs,
@@ -83,11 +82,6 @@ GtkArg*               gtk_object_collect_args  (guint   *nargs,
 
 static guint object_signals[LAST_SIGNAL] = { 0 };
 
-static GHashTable *object_data_ht = NULL;
-static GMemChunk *object_data_mem_chunk = NULL;
-static GSList *object_data_id_list = NULL;
-static guint object_data_id_index = 0;
-
 static GHashTable *arg_info_ht = NULL;
 
 static const gchar *user_data_key = "user_data";
@@ -95,6 +89,19 @@ static guint user_data_key_id = 0;
 static const gchar *weakrefs_key = "gtk-weakrefs";
 static guint weakrefs_key_id = 0;
 
+static GHashTable *object_data_ht = NULL;
+static GSList *object_data_id_list = NULL;
+static guint object_data_id_index = 0;
+static GtkObjectData *gtk_object_data_free_list = NULL;
+
+#define GTK_OBJECT_DATA_DESTROY( odata )	{ \
+  if (odata->destroy) \
+    odata->destroy (odata->data); \
+  odata->next = gtk_object_data_free_list; \
+  gtk_object_data_free_list = odata; \
+}
+
+
 #ifdef G_ENABLE_DEBUG
 static guint obj_count = 0;
 static GHashTable *living_objs_ht = NULL;
@@ -135,7 +142,7 @@ gtk_object_pointer_hash (const gpointer v)
  ****************************************************/
 
 void
-gtk_object_init_type ()
+gtk_object_init_type (void)
 {
   GtkType object_type = 0;
   GtkTypeInfo object_info =
@@ -159,7 +166,7 @@ gtk_object_init_type ()
 }
 
 GtkType
-gtk_object_get_type ()
+gtk_object_get_type (void)
 {
   return GTK_TYPE_OBJECT;
 }
@@ -224,7 +231,6 @@ gtk_object_init (GtkObject *object)
  * Functions to end a GtkObject's life time
  *
  ********************************************/
-
 void
 gtk_object_destroy (GtkObject *object)
 {
@@ -253,22 +259,22 @@ gtk_object_shutdown (GtkObject *object)
 static void
 gtk_object_real_destroy (GtkObject *object)
 {
-  gtk_signal_handlers_destroy (object);
+  if (GTK_OBJECT_CONNECTED (object))
+    gtk_signal_handlers_destroy (object);
 }
 
 static void
 gtk_object_finalize (GtkObject *object)
 {
-  GtkObjectData *odata, *next;
-  
   gtk_object_notify_weaks (object);
 
-  odata = object->object_data;
-  while (odata)
+  while (object->object_data)
     {
-      next = odata->next;
-      gtk_object_data_destroy (odata);
-      odata = next;
+      GtkObjectData *odata;
+
+      odata = object->object_data;
+      object->object_data = odata->next;
+      GTK_OBJECT_DATA_DESTROY (odata);
     }
   
   g_free (object);
@@ -934,9 +940,19 @@ gtk_object_get_arg_type (const gchar *arg_name)
   gchar buffer[1024];
   gchar *t;
 
+  g_return_val_if_fail (arg_name != NULL, 0);
+
   if (!arg_info_ht)
     return GTK_TYPE_INVALID;
 
+  if (!arg_name || strlen (arg_name) > 1000)
+    {
+      /* security audit
+       */
+      g_warning ("gtk_object_get_arg_type(): argument `arg_name' exceeds maximum size.");
+      return 0;
+    }
+
   t = strchr (arg_name, ':');
   if (!t || (t[0] != ':') || (t[1] != ':'))
     {
@@ -991,16 +1007,17 @@ gtk_object_set_data_by_id_full (GtkObject        *object,
 				GtkDestroyNotify  destroy)
 {
   GtkObjectData *odata;
-  GtkObjectData *prev;
   
   g_return_if_fail (object != NULL);
   g_return_if_fail (GTK_IS_OBJECT (object));
   g_return_if_fail (data_id > 0);
-  
+
+  odata = object->object_data;
   if (!data)
     {
+      GtkObjectData *prev;
+      
       prev = NULL;
-      odata = object->object_data;
       
       while (odata)
 	{
@@ -1008,10 +1025,10 @@ gtk_object_set_data_by_id_full (GtkObject        *object,
 	    {
 	      if (prev)
 		prev->next = odata->next;
-	      if (odata == object->object_data)
+	      else
 		object->object_data = odata->next;
 	      
-	      gtk_object_data_destroy (odata);
+	      GTK_OBJECT_DATA_DESTROY (odata);
 	      break;
 	    }
 	  
@@ -1021,28 +1038,58 @@ gtk_object_set_data_by_id_full (GtkObject        *object,
     }
   else
     {
-      odata = object->object_data;
+      GtkObjectData *prev;
+      
+      prev = NULL;
+
       while (odata)
 	{
 	  if (odata->id == data_id)
 	    {
+	      /* we need to be unlinked while invoking the destroy function
+	       */
 	      if (odata->destroy)
-		odata->destroy (odata->data);
+		{
+		  if (prev)
+		    prev->next = odata->next;
+		  else
+		    object->object_data = odata->next;
+		  
+		  odata->destroy (odata->data);
+		  
+		  odata->next = object->object_data;
+		  object->object_data = odata;
+		}
 	      
 	      odata->data = data;
 	      odata->destroy = destroy;
 	      return;
 	    }
 	  
-	  odata = odata->next;
+	  prev = odata;
+	  odata = prev->next;
 	}
       
-      if (!object_data_mem_chunk)
-	object_data_mem_chunk = g_mem_chunk_new ("object data mem chunk",
-						 sizeof (GtkObjectData),
-						 1024, G_ALLOC_AND_FREE);
+      if (gtk_object_data_free_list)
+	{
+	  odata = gtk_object_data_free_list;
+	  gtk_object_data_free_list = odata->next;
+	}
+      else
+	{
+	  GtkObjectData *odata_block;
+	  guint i;
+
+	  odata_block = g_new0 (GtkObjectData, GTK_OBJECT_DATA_BLOCK_SIZE);
+	  for (i = 1; i < GTK_OBJECT_DATA_BLOCK_SIZE; i++)
+	    {
+	      (odata_block + i)->next = gtk_object_data_free_list;
+	      gtk_object_data_free_list = (odata_block + i);
+	    }
+
+	  odata = odata_block;
+	}
       
-      odata = g_chunk_new (GtkObjectData, object_data_mem_chunk);
       odata->id = data_id;
       odata->data = data;
       odata->destroy = destroy;
@@ -1159,27 +1206,16 @@ gtk_object_remove_data (GtkObject   *object,
     gtk_object_set_data_by_id_full (object, id, NULL, NULL);
 }
 
-static void
-gtk_object_data_destroy (GtkObjectData *odata)
-{
-  g_return_if_fail (odata != NULL);
-
-  if (odata->destroy)
-    odata->destroy (odata->data);
-
-  g_mem_chunk_free (object_data_mem_chunk, odata);
-}
-
 static guint*
-gtk_object_data_id_alloc ()
+gtk_object_data_id_alloc (void)
 {
   static guint next_id = 1;
   guint *ids;
 
   if (!object_data_id_list ||
-      (object_data_id_index == OBJECT_DATA_ID_CHUNK))
+      (object_data_id_index == GTK_OBJECT_DATA_ID_BLOCK_SIZE))
     {
-      ids = g_new (guint, OBJECT_DATA_ID_CHUNK);
+      ids = g_new (guint, GTK_OBJECT_DATA_ID_BLOCK_SIZE);
       object_data_id_index = 0;
       object_data_id_list = g_slist_prepend (object_data_id_list, ids);
     }
@@ -1556,46 +1592,42 @@ gtk_object_unref (GtkObject *object)
     }
 }
 
-
-#ifdef G_ENABLE_DEBUG
 static GtkObject *gtk_trace_object = NULL;
 void
-gtk_trace_referencing (gpointer    *o,
+gtk_trace_referencing (GtkObject   *object,
 		       const gchar *func,
-		       guint	   local_frame,
+		       guint	   dummy,
 		       guint	   line,
 		       gboolean	   do_ref)
 {
-  gboolean exists;
-  GtkObject *object = (GtkObject*) o;
-
   if (gtk_debug_flags & GTK_DEBUG_OBJECTS)
     {
+      gboolean exists = TRUE;
+  
       g_return_if_fail (object != NULL);
       g_return_if_fail (GTK_IS_OBJECT (object));
-
+      
+#ifdef	G_ENABLE_DEBUG
       exists = g_hash_table_lookup (living_objs_ht, object) != NULL;
+#endif	/* G_ENABLE_DEBUG */
       
       if (exists &&
 	  (object == gtk_trace_object ||
 	   gtk_trace_object == (void*)42))
-	printf ("trace: object_%s: (%s:%p)->ref_count=%d%s (%s_f%02d:%d)\n",
-		do_ref ? "ref" : "unref",
-		gtk_type_name (GTK_OBJECT_TYPE (object)),
-		object,
-		object->ref_count,
-		do_ref ? " + 1" : " - 1 ",
-		func,
-		local_frame,
-		line);
-  
-      if (!exists)
-	printf ("trace: object_%s(%p): no such object! (%s_f%02d:%d)\n",
-		do_ref ? "ref" : "unref",
-		object,
-		func,
-		local_frame,
-		line);
+	fprintf (stdout, "trace: object_%s: (%s:%p)->ref_count=%d %s (%s:%d)\n",
+		 do_ref ? "ref" : "unref",
+		 gtk_type_name (GTK_OBJECT_TYPE (object)),
+		 object,
+		 object->ref_count,
+		 do_ref ? "+ 1" : "- 1",
+		 func,
+		 line);
+      else if (!exists)
+	fprintf (stdout, "trace: object_%s(%p): no such object! (%s:%d)\n",
+		 do_ref ? "ref" : "unref",
+		 object,
+		 func,
+		 line);
     }
   
   if (do_ref)
@@ -1603,5 +1635,3 @@ gtk_trace_referencing (gpointer    *o,
   else
     gtk_object_unref (object);
 }
-#endif /* G_ENABLE_DEBUG */
-
